@@ -343,42 +343,7 @@ class Axes:
         self._pad.Update()  # Updating the pad prevents spontaneous seg faults...
 
         # Apply formatting (if any) before calling `Draw()`
-
-        # Marker attributes
-        if "markercolor" in kwargs and "markeralpha" not in kwargs:
-            obj.SetMarkerColor(kwargs["markercolor"])
-
-        if "markercolor" in kwargs and "markeralpha" in kwargs:
-            obj.SetMarkerColorAlpha(kwargs["markercolor"], kwargs["markeralpha"])
-
-        if "markersize" in kwargs:
-            obj.SetMarkerSize(kwargs["markersize"])
-
-        if "markerstyle" in kwargs:
-            obj.SetMarkerStyle(kwargs["markerstyle"])
-
-        # Line attributes
-        if "linecolor" in kwargs and "linealpha" not in kwargs:
-            obj.SetLineColor(kwargs["linecolor"])
-
-        if "linecolor" in kwargs and "linealpha" in kwargs:
-            obj.SetLineColorAlpha(kwargs["linecolor"], kwargs["linealpha"])
-
-        if "linewidth" in kwargs:
-            obj.SetLineWidth(kwargs["linewidth"])
-
-        if "linestyle" in kwargs:
-            obj.SetLineStyle(kwargs["linestyle"])
-
-        # Fill attributes
-        if "fillcolor" in kwargs and "fillalpha" not in kwargs:
-            obj.SetFillColor(kwargs["fillcolor"])
-
-        if "fillcolor" in kwargs and "fillalpha" in kwargs:
-            obj.SetFillColorAlpha(kwargs["fillcolor"], kwargs["fillalpha"])
-
-        if "fillstyle" in kwargs:
-            obj.SetFillStyle(kwargs["fillstyle"])
+        root_helpers.set_graphics_attributes(obj, **kwargs)
 
         # Get current axis limits
         old_left, old_right = self.get_xlim()
@@ -480,6 +445,56 @@ class Axes:
 
         self._is_empty = False  # Record that the axes are no longer empty
 
+    def plot2d(self, obj, options="", label=None, labelfmt=None, **kwargs):
+        """Plot object with two independent variables on these axes.
+
+        Here, `obj` is normally a TH2 histogram, although this is not required. Unlike
+        :meth:`~.core.Axes.plot`, where an object is plotted on an already-existing
+        frame, this function takes the object itself as the axes' "frame". This is
+        always the case unless the "SAME" option is used to plot `obj`; here, the axes'
+        frame is not replaced with `obj`. If the axes' frame already exists as a TH1F
+        object (the default when an :class:`~.core.Axes` object is created), it is
+        deleted and replaced with `obj`.
+
+        See :meth:`~.core.Axes.plot` for full documentation of the other options.
+        """
+        self._pad.cd()
+        self._pad.Update()  # Updating the pad prevents spontaneous seg faults...
+
+        # Apply formatting (if any) before calling `Draw()`
+        root_helpers.set_graphics_attributes(obj, **kwargs)
+
+        # Draw the object, depending on its type
+        if isinstance(obj, root.TH2):
+            if isinstance(self._frame, root.TH1F):
+                if not self._is_empty:
+                    warnings.warn("plot2d: overwriting non-empty axes")
+
+                self._frame.Delete()
+                self._frame = obj
+
+            elif "SAME" not in options.upper():
+                self._frame = obj
+
+            obj.Draw(options)
+
+        else:
+            try:
+                warnings.warn(
+                    "plot2d: attempting to plot an object that is not a TH2.\n"
+                    "This may result in unexpected behaviour."
+                )
+                obj.Draw(options)
+
+            except AttributeError:
+                raise TypeError("Attempting to plot an object with no Draw() method")
+
+        # Add object to list of legend entries if label was provided
+        if label is not None:
+            self._legend_entries.append((obj, label, labelfmt))
+
+        self._is_empty = False  # Record that the axes are no longer empty
+
     def graph(self, x, y, xerr=None, yerr=None, options="P", **kwargs):
         """Create and plot a ROOT TGraph from array-like input.
 
@@ -651,6 +666,59 @@ class Axes:
             self._pad.Modified()
             self._logy = True
 
+    def set_zscale(self, value):
+        """
+        Set the z-axis scale.
+
+        Parameters
+        ----------
+        value : {"linear", "log"}
+            The axis scale type to apply.
+        """
+        if value in ["linear", "lin"]:
+            self._pad.SetLogz(0)
+            self._logz = False
+
+        elif value in ["log", "logy"]:
+            bottom, top = self.get_zlim()
+            if top <= 0:
+                warnings.warn(
+                    "Current frame has no positive values, and therefore cannot "
+                    "be log-scaled. Try running ax.set_zlim() first."
+                )
+            elif bottom <= 0:
+                # Arbitrarily set bottom to 0.1 (or 0.1*top if top < 0.1)
+                # so that the frame can be displayed
+                if top <= 0.1:
+                    self.set_zlim(bottom=0.1 * top)
+                else:
+                    self.set_zlim(bottom=0.1)
+
+            self._pad.cd()
+            self._pad.SetLogz(1)
+            self._pad.Modified()
+            self._logz = True
+
+    def set_pad_margins(self, left=None, right=None, bottom=None, top=None):
+        """Set the pad margin between the axes and the outer edge of the pad.
+
+        Parameters
+        ----------
+        left, right, bottom, top : float, optional
+            Margin sizes in NDC units [0, 1]. The default is None (use default margin).
+        """
+        if left is not None:
+            self.pad.SetLeftMargin(left)
+
+        if right is not None:
+            self.pad.SetRightMargin(right)
+
+        if bottom is not None:
+            self.pad.SetBottomMargin(bottom)
+
+        if top is not None:
+            self.pad.SetTopMargin(top)
+
     def add_margins(self, left=0.0, right=0.0, bottom=0.0, top=0.0):
         """Adds margins between the axes and the data within.
 
@@ -717,7 +785,7 @@ class Axes:
         """
         return self._frame.GetXaxis().GetTitle()
 
-    def set_xlabel(self, xlabel, loc=None):
+    def set_xlabel(self, xlabel, loc=None, **kwargs):
         """Set the label for the x-axis.
 
         Parameters
@@ -727,6 +795,16 @@ class Axes:
         loc : {'center', 'right'}
             The label position. The default is the same as ROOT's default x-axis
             title position (right).
+        **kwargs : formatting options, optional
+            `kwargs` are used to specify the axis attributes. See the ROOT docs for
+            available options:
+
+                | https://root.cern.ch/doc/master/classTAttAxis.html
+
+            The kwarg syntax is the same as the equivalent ROOT attribute setter
+            function, but in all lower case and without the 'Set' prefix. For example,
+            to set the axis title size, use ``titlesize=...``, which calls
+            ``SetTitleSize(...)``.
         """
         self._frame.GetXaxis().SetTitle(xlabel)
 
@@ -738,6 +816,8 @@ class Axes:
                 "supported values are {{'center', 'right'}}".format(loc)
             )
 
+        root_helpers.set_axis_attributes(self._frame.GetXaxis(), **kwargs)
+
         self._pad.Modified()  # Draw the updated axes
 
     def get_ylabel(self):
@@ -745,7 +825,7 @@ class Axes:
         """
         return self._frame.GetYaxis().GetTitle()
 
-    def set_ylabel(self, ylabel, loc=None):
+    def set_ylabel(self, ylabel, loc=None, **kwargs):
         """Set the label for the y-axis.
 
         Parameters
@@ -755,6 +835,8 @@ class Axes:
         loc : {'center', 'top'}
             The label position. The default is the same as ROOT's default y-axis
             title position (top).
+        **kwargs : formatting options, optional
+            See :meth:`~.core.Axes.set_xlabel` for usage.
         """
         self._frame.GetYaxis().SetTitle(ylabel)
 
@@ -766,6 +848,40 @@ class Axes:
                 "supported values are {{'center', 'top'}}".format(loc)
             )
 
+        root_helpers.set_axis_attributes(self._frame.GetYaxis(), **kwargs)
+
+        self._pad.Modified()  # Draw the updated axes
+
+    def get_zlabel(self):
+        """Get the zlabel text string.
+        """
+        return self._frame.GetZaxis().GetTitle()
+
+    def set_zlabel(self, zlabel, loc=None, **kwargs):
+        """Set the label for the z-axis.
+
+        Parameters
+        ----------
+        zlabel : str
+            The label text.
+        loc : {'center', 'top'}
+            The label position. The default is the same as ROOT's default z-axis
+            title position (top).
+        **kwargs : formatting options, optional
+            See :meth:`~.core.Axes.set_xlabel` for usage.
+        """
+        self._frame.GetZaxis().SetTitle(zlabel)
+
+        if loc in ["center", "centre"]:
+            self._frame.GetZaxis().CenterTitle()
+        elif loc is not None and loc != "top":
+            raise ValueError(
+                "'{}' is not a valid value for 'loc'; "
+                "supported values are {{'center', 'top'}}".format(loc)
+            )
+
+        root_helpers.set_axis_attributes(self._frame.GetZaxis(), **kwargs)
+
         self._pad.Modified()  # Draw the updated axes
 
     def get_xlim(self):
@@ -786,7 +902,23 @@ class Axes:
         bottom, top : (float, float)
             The current y-axis limits in data coordinates.
         """
-        return (self._frame.GetMinimum(), self._frame.GetMaximum())
+        if isinstance(self._frame, root.TH1F):
+            return (self._frame.GetMinimum(), self._frame.GetMaximum())
+        else:
+            return (self._frame.GetYaxis().GetXmin(), self._frame.GetYaxis().GetXmax())
+
+    def get_zlim(self):
+        """Returns the frame's z-axis limits.
+
+        Returns
+        -------
+        bottom, top : (float, float)
+            The current z-axis limits in data coordinates.
+        """
+        if isinstance(self._frame, root.TH2F):
+            return (self._frame.GetMinimum(), self._frame.GetMaximum())
+        else:
+            return (self._frame.GetZaxis().GetXmin(), self._frame.GetZaxis().GetXmax())
 
     def set_xlim(self, left=None, right=None):
         """Sets the frame's x-axis limits.
@@ -846,7 +978,26 @@ class Axes:
         if left > right:
             raise ValueError("Axis limits must be in increasing order")
 
-        self._frame.GetXaxis().SetLimits(left, right)
+        if right <= 0 and self._logx:
+            warnings.warn(
+                "Attempting to set non-positive right xlim on a log-scaled axis.\n"
+                "Invalid limit will be ignored.",
+                stacklevel=2,
+            )
+            right = self.get_xlim()[1]
+
+        elif left <= 0 and self._logx:
+            warnings.warn(
+                "Attempting to set non-positive left xlim on a log-scaled axis.\n"
+                "Invalid limit will be ignored.",
+                stacklevel=2,
+            )
+            left = self.get_xlim()[0]
+
+        if isinstance(self._frame, root.TH1F):
+            self._frame.GetXaxis().SetLimits(left, right)
+        else:
+            self._frame.GetXaxis().SetRangeUser(left, right)
 
         self._pad.Modified()  # Draw the updated axes
 
@@ -905,6 +1056,78 @@ class Axes:
         if bottom > top:
             raise ValueError("Axis limits must be in increasing order")
 
+        if top <= 0 and self._logy:
+            warnings.warn(
+                "Attempting to set non-positive top ylim on a log-scaled axis.\n"
+                "Invalid limit will be ignored.",
+                stacklevel=2,
+            )
+            top = self.get_ylim()[1]
+
+        elif bottom <= 0 and self._logy:
+            warnings.warn(
+                "Attempting to set non-positive bottom ylim on a log-scaled axis.\n"
+                "Invalid limit will be ignored.",
+                stacklevel=2,
+            )
+            bottom = self.get_ylim()[0]
+
+        if isinstance(self._frame, root.TH1F):
+            self._frame.SetMinimum(bottom)
+            self._frame.SetMaximum(top)
+        else:
+            self._frame.GetYaxis().SetRangeUser(bottom, top)
+
+        self._pad.Modified()  # Draw the updated axes
+
+        return (bottom, top)
+
+    def set_zlim(self, bottom=None, top=None):
+        """Sets the frame's z-axis limits.
+
+        See See :meth:`~.core.Axes.set_ylim` for full documentation.
+        """
+        if isinstance(self._frame, root.TH1F):
+            warnings.warn("Attempting to set z-axis limits for 2D axes")
+            return
+
+        if top is None and np.iterable(bottom):
+            bottom, top = bottom
+
+        if bottom is None or top is None:
+            old_bottom, old_top = self.get_zlim()
+            if bottom is None:
+                bottom = old_bottom
+            if top is None:
+                top = old_top
+
+        if bottom == top:
+            warnings.warn(
+                "Attempting to set identical bottom == top == {} z-axis limits".format(
+                    bottom
+                ),
+                stacklevel=2,
+            )
+
+        if bottom > top:
+            raise ValueError("Axis limits must be in increasing order")
+
+        if top <= 0 and self._logz:
+            warnings.warn(
+                "Attempting to set non-positive top zlim on a log-scaled axis.\n"
+                "Invalid limit will be ignored.",
+                stacklevel=2,
+            )
+            top = self.get_zlim()[1]
+
+        elif bottom <= 0 and self._logy:
+            warnings.warn(
+                "Attempting to set non-positive bottom zlim on a log-scaled axis.\n"
+                "Invalid limit will be ignored.",
+                stacklevel=2,
+            )
+            bottom = self.get_zlim()[0]
+
         self._frame.SetMinimum(bottom)
         self._frame.SetMaximum(top)
 
@@ -941,6 +1164,8 @@ class Axes:
             that some formatting options may not work if the ATLAS style
             settings have been applied.
         """
+        self._pad.cd()
+
         if self._legend is not None and isinstance(self._legend, root.TLegend):
             warnings.warn("These axes already have a legend, will overwrite", stacklevel=2)
             self._legend.Delete()
@@ -951,47 +1176,8 @@ class Axes:
         # Do this here since this option is not available in the `TStyle` class
         self._legend.SetFillColorAlpha(0, 0)
 
-        # Line attributes
-        if "linecolor" in kwargs and "linealpha" not in kwargs:
-            self._legend.SetLineColor(kwargs["linecolor"])
-
-        if "linecolor" in kwargs and "linealpha" in kwargs:
-            self._legend.SetLineColorAlpha(kwargs["linecolor"], kwargs["linealpha"])
-
-        if "linewidth" in kwargs:
-            self._legend.SetLineWidth(kwargs["linewidth"])
-
-        if "linestyle" in kwargs:
-            self._legend.SetLineStyle(kwargs["linestyle"])
-
-        # Fill attributes
-        if "fillcolor" in kwargs and "fillalpha" not in kwargs:
-            self._legend.SetFillColor(kwargs["fillcolor"])
-
-        if "fillcolor" in kwargs and "fillalpha" in kwargs:
-            self._legend.SetFillColorAlpha(kwargs["fillcolor"], kwargs["fillalpha"])
-
-        if "fillstyle" in kwargs:
-            self._legend.SetFillStyle(kwargs["fillstyle"])
-
-        # Text attributes
-        if "textsize" in kwargs:
-            self._legend.SetTextSize(kwargs["textsize"])
-
-        if "textfont" in kwargs:
-            self._legend.SetTextFont(kwargs["textfont"])
-
-        if "textalign" in kwargs:
-            self._legend.SetTextAlign(kwargs["textalign"])
-
-        if "textcolor" in kwargs and "textalpha" not in kwargs:
-            self._legend.SetTextColor(kwargs["textcolor"])
-
-        if "textcolor" in kwargs and "textalpha" in kwargs:
-            self._legend.SetTextColorAlpha(kwargs["textcolor"], kwargs["textalpha"])
-
-        if "textangle" in kwargs:
-            self._legend.SetTextAngle(kwargs["textangle"])
+        # Set graphics attributes
+        root_helpers.set_graphics_attributes(self._legend, **kwargs)
 
         # Columns
         if "ncol" in kwargs:
